;==============================================================================
; File: ALCD12C.S
; Func: HC12compact Driver Routines for alphanumerical LC-Displays
;       (HD44780 based, from 1x8 up to 2x40/4x20 characters)
; Copr: (C)1998 by MCT, Oliver Thamm
; Vers: 1.4
; Rem.: The bus timing is rather critical, so proper operation will depend on
;       the particular LCD type used! To avoid trouble we use an additional
;       delay after each LCD operation. The timing is derived from the ECLK.
;==============================================================================

;	.include "hc12.i"

;-- local data ----------------------------------------------------------------

			.area bss
_PosXY::
PosX:		.blkb 1		; current cursor position X
PosY:		.blkb 1		; current cursor position Y
MaxX:		.blkb 1		; number of characters per line (8..40)
MaxY:		.blkb 1		; number of lines (1,2 or 4)


;------------------------------------------------------------------------------
; Func: LCD definitions
; Rem.: Modify this if yor LC-Display uses another scheme!
;------------------------------------------------------------------------------

			.area text

RowAddrTable::
			.byte $00	; DDRAM address for 1st line
			.byte $40	; DDRAM address for 2nd line
			.byte $10 	; DDRAM address for 3rd line
			.byte $50 	; DDRAM address for 4th line


;------------------------------------------------------------------------------
; Func: Target dependend hardware access functions
; Rem.: Modify this according to the actual bus interface!
;------------------------------------------------------------------------------

LCD_CMD  = $0381
LCD_DATA = $0383


;-- write a command/data byte -------------------------------------------------


__writeCmd:
			brset LCD_CMD,$80,__writeCmd
			nop
			staa  LCD_CMD
			bsr   __wt_50ys
			cmpa  #$01					; "Clear" command needs up to 1.64ms!
			beq   __wt_2ms				; (exit via subroutine)
			rts

;----------------

__writeData:
			brset LCD_CMD,$80,__writeData
			nop
			staa  LCD_DATA
			bra   __wt_50ys				; (exit via subroutine)

;----------------

__wt_2ms:	pshx 
			ldx  #5333					; 5333 x 375ns = 2ms
			bra  __wt_l1
			;
__wt_50ys:	pshx
			ldx  #133					; 133 x 375ns = 50ys
__wt_l1:	dbne x,__wt_l1				; [3] x 125ns = 375ns
			pulx
			rts

;------------------------------------------------------------------------------
; void initLCD(int columns, int lines);
; void clearLCD(void);
;
; Func: Initialize LCD
; Args: columns (D) -> MaxX, lines (2,sp) -> MaxY
; Retn: -
;
_initLCD::	stab MaxX
			movb 3,sp,MaxY
_clearLCD::
			psha
			pshy
			ldy  #__iniTableStart
__iniCmdLoop:
			ldaa 0,y
			jsr  __writeCmd
			iny
			cpy  #__iniTableEnd
			bne  __iniCmdLoop
			clr  PosX
			clr  PosY
			puly
			pula
			rts

__iniTableStart:
			.byte $38	; Function Set (8Bit,5x7)
			.byte $38	; dito
			.byte $0C	; Display on, Cursor off, Blink off
			.byte $01	; Clear display
			.byte $06	; Entry Set: Increment, No shift
			.byte $80	; Reset Address counter
__iniTableEnd:


;------------------------------------------------------------------------------
; void gotoxyLCD(int x, int y);
;
; Func: Set cursor to location x,y
; Args:    D -> x = (0...MaxX-1)
;       2,SP -> y = (0...MaxY-1)
; Retn: -
;
_gotoxyLCD::
			tfr  b,a			; A = x
			ldab 3,sp			; B = y
			;
__gotoxyAB:	pshd
			pshx
__doPos:	std  _PosXY
			ldx  #RowAddrTable
			adda B,X
			oraa #$80
			jsr  __writeCmd
			pulx
			puld
			rts


;-- increment cursor position -------------------------------------------------

__incPos:	pshd
			pshx
			ldd  _PosXY
			inca
			cmpa MaxX
			bne  __goPos
			clra
			incb
			cmpb MaxY
			bne  __goPos
			clrb
__goPos:	bra  __doPos


;-- decrement cursor position -------------------------------------------------

__decPos:	pshd
			pshx
			ldd  _PosXY
			tsta
			bne  __canBackX
			ldaa MaxX
			tstb
			bne  __canBackY
			ldab MaxY
__canBackY:	decb
__canBackX:	deca
			bra  __doPos


;------------------------------------------------------------------------------
; void putLCD(char c);
;
; Func: Display a character considering control codes and
;       translate certain codes
; Args: D = character
; Retn: -
;
_putLCD::	exg  a,b
__putLCDa:	jsr  __doControls
			tsta
			beq  __wasCtrl
			jsr  __translateChr
			jsr  __writeData
			jsr  __incPos
__wasCtrl:	rts


;-- control characters --------------------------------------------------------

__doBS:		jsr  __decPos
			ldaa #$20
			jsr  __putLCDa
			jsr  __decPos
			clra
			rts

__doCRLF:	ldd  _PosXY
			incb
			cmpb MaxY
			bne  __doCRx
			clrb
			bra  __doCRx

__doCR:		ldab PosY
__doCRx:	clra
			jsr  __gotoxyAB
			rts

__doClrLine:
			jsr  __doCR
			ldab PosY
			pshb
			ldab MaxX
			ldaa #$20
__clrLnLoo:	jsr  __putLCDa
			decb
			bne  __clrLnLoo
			pulb
			clra
			jsr  __gotoxyAB
			rts

__doClrScr:	jsr  _clearLCD
			clra
			rts

__doCursorOn:
			ldaa #$0F
			bra  __switchCursor
__doCursorOff:
			ldaa #$0C
__switchCursor:
			jsr  __writeCmd
			clra
			rts


;-- check for control character -----------------------------------------------

__doControls:
			cmpa #$08			; backspace
			beq  __doBS
			cmpa #$0A			; linefeed (+carriage return)
			beq  __doCRLF
			cmpa #$0B			; clear line
			beq  __doClrLine
			cmpa #$0C			; clear screen
			beq  __doClrScr
			cmpa #$0D			; carriage return
			beq  __doCR
			cmpa #$0E			; cursor on
			beq  __doCursorOn
			cmpa #$0F			; cursor off
			beq  __doCursorOff
			rts


;-- translate certain character codes -----------------------------------------

__translateChr:
			ldy  #__transTableS
__anotherEntry:
			cmpa 0,y
			bne  __didntFind
			ldaa 1,y
			rts
__didntFind:
			iny
			iny
			cpy  #__transTableE
			bne  __anotherEntry
			tsta
			bmi  __replaceBlank
			cmpa #$20
			bhs  __xTrans
__replaceBlank:
			ldaa #$20
__xTrans:	rts

__transTableS:
			.byte   ',$E1
			.byte   ',$E1
			.byte   ',$EF
			.byte   ',$EF
			.byte   ',$F5
			.byte   ',$F5
			.byte   ',$E2
			.byte   $5C,$A4 	; Backslash
			.byte   $7E,$F3		; Tilde
			.byte   $7F,$FF		; DEL
__transTableE:


;-- fine ----------------------------------------------------------------------
